/* SPDX-License-Identifier: GPL-2.0 */
/*
 * wireless_firmware.h
 *
 * wireless firmware driver
 *
 * Copyright (c) 2020-2021 Huawei Technologies Co., Ltd.
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 */

#ifndef _WIRELESS_FIRMWARE_H_
#define _WIRELESS_FIRMWARE_H_

#include <linux/types.h>
#include <linux/device.h>
#include <chipset_common/hwpower/common_module/power_firmware.h>

#define WIRELESS_FW_DFT_IC_NUMS                 1

#define WIRELESS_FW_PROGRAMED                   1
#define WIRELESS_FW_NON_PROGRAMED               0
#define WIRELESS_FW_ERR_PROGRAMED               2
#define WIRELESS_FW_WORK_DELAYED_TIME           500 /* ms */

#define WIRELESS_PROGRAM_FW                     0
#define WIRELESS_RECOVER_FW                     1

enum wireless_fw_sysfs_type {
	WLFW_SYSFS_BEGIN = 0,
	WLFW_SYSFS_PROGRAM_FW = WLFW_SYSFS_BEGIN,
	WLFW_SYSFS_CHK_FW,
	WLFW_SYSFS_END,
};

struct wireless_fw_ops {
	void *dev_data;
	struct list_head fw_node;
	const char *ic_name;
	int (*get_fw_status)(unsigned int *status, void *dev_data);
	int (*program_fw)(unsigned int prog_type, void *dev_data);
	int (*check_fw)(void *dev_data);
	ssize_t (*read_fw)(void *dev_data, char *buf, size_t size);
	ssize_t (*write_fw)(void *dev_data, const char *buf, size_t size);
	bool (*is_mtp_chk_complete)(void *dev_data);
};

struct wireless_fw_dev {
	struct device *dev;
	struct work_struct program_work;
	struct wireless_fw_ops *ops;
	bool program_fw_flag;
	int ic_nums;
};

static inline int wireless_fw_ops_register(struct wireless_fw_ops *ops, unsigned int ic_type)
{
	return 0;
}

static inline bool wireless_fw_is_mtp_chk_complete(void)
{
	return true;
}


#endif /* _WIRELESS_FIRMWARE_H_ */
