/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2016-2019. All rights reserved.
 * Description: block header
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef MAS_BLK_INTERNAL_H
#define MAS_BLK_INTERNAL_H
#include <linux/bio.h>
#include <linux/blk-mq.h>
#include <linux/blk_types.h>
#include <linux/blkdev.h>
#include <platform_include/basicplatform/linux/rdr_platform.h>
#include <platform_include/basicplatform/linux/hck/ufs/hck_dfa_mas_mcq.h>
#ifdef CONFIG_MAS_MCQ
#include "mas_mcq.h"
#endif

#define IO_FROM_SUBMIT_BIO_MAGIC 0x4C
#define IO_FROM_BLK_EXEC 0x4D

#define BLK_LLD_INIT_MAGIC 0x7A
#define IO_BUF_LEN 10

#define TEN_MS 10
#define ONE_HUNDRED 100

#define REQ_FG_META (REQ_FG | REQ_META)

/* Used to device sqr sqw bw */
#define BLK_MID_SQW_BW  300
#define BLK_MAX_SQW_BW  500
#define BLK_MAX_SQR_BW  0
#define BLK_MAX_SQW_BW_3_1 995
#define BLK_MAX_SQR_BW_3_1 1800
#define DEVICE_CAPACITY_128_G  0x10000000  /* 128G/512 */
#define DEVICE_CAPACITY_256_G  0x20000000

#ifdef CONFIG_MAS_UNISTORE_PRESERVE
#define SECTOR_BYTE 9
#define SECTION_SECTOR 3
#define BLKSIZE 4096 /* bytes */
#define RESERVED_RESET_TAG_NUM 4

struct unistore_section_info {
	struct list_head section_list;
	sector_t section_start_lba;
	ktime_t section_insert_time;
	sector_t section_id;
	sector_t next_section_start_lba;
	sector_t next_section_id;
	bool slc_mode;
	bool rcv_io_complete_flag;
};
#endif

enum blk_lld_feature_bits {
	__BLK_LLD_DUMP_SUPPORT = 0,
	__BLK_LLD_LATENCY_SUPPORT,
	__BLK_LLD_FLUSH_REDUCE_SUPPORT,
	__BLK_LLD_BUSYIDLE_SUPPORT,
	__BLK_LLD_IOSCHED_UFS_MQ,
	__BLK_LLD_IOSCHED_UFS_HW_IDLE,
	__BLK_LLD_UFS_CP_EN,
	__BLK_LLD_UFS_ORDER_EN,
	__BLK_LLD_UFS_UNISTORE_EN,
	__BLK_LLD_IOSCHED_MMC_MQ,
	__BLK_LLD_UFS_CPPLUS_EN,
	__BLK_LLD_IOSCHED_MAS_MCQ,
};

#define BLK_LLD_DUMP_SUPPORT (1ULL << __BLK_LLD_DUMP_SUPPORT)
#define BLK_LLD_LATENCY_SUPPORT (1ULL << __BLK_LLD_LATENCY_SUPPORT)
#define BLK_LLD_FLUSH_REDUCE_SUPPORT (1ULL << __BLK_LLD_FLUSH_REDUCE_SUPPORT)
#define BLK_LLD_BUSYIDLE_SUPPORT (1ULL << __BLK_LLD_BUSYIDLE_SUPPORT)
#define BLK_LLD_IOSCHED_UFS_MQ (1ULL << __BLK_LLD_IOSCHED_UFS_MQ)
#define BLK_LLD_IOSCHED_UFS_HW_IDLE (1ULL << __BLK_LLD_IOSCHED_UFS_HW_IDLE)
#define BLK_LLD_UFS_CP_EN (1ULL << __BLK_LLD_UFS_CP_EN)
#define BLK_LLD_UFS_ORDER_EN (1ULL << __BLK_LLD_UFS_ORDER_EN)
#define BLK_LLD_UFS_UNISTORE_EN (1ULL << __BLK_LLD_UFS_UNISTORE_EN)
#define BLK_LLD_IOSCHED_MMC_MQ (1ULL << __BLK_LLD_IOSCHED_MMC_MQ)
#define BLK_LLD_UFS_CPPLUS_EN (1ULL << __BLK_LLD_UFS_CPPLUS_EN)
#define BLK_LLD_IOSCHED_MAS_MCQ (1ULL << __BLK_LLD_IOSCHED_MAS_MCQ)

#define BLK_QUEUE_DURATION_UNIT_NS 10000

enum blk_busyidle_nb_flag {
	BLK_BUSYIDLE_NB_NOT_JOIN_POLL = 0,
};

#define BLK_BUSYIDLE_NB_FLAG_NOT_JOIN_POLL                                    \
	(1 << BLK_BUSYIDLE_NB_NOT_JOIN_POLL)

enum iosched_strategy {
	IOSCHED_NONE = 0,
	IOSCHED_MAS_UFS_MQ,
	IOSCHED_MAS_MMC_MQ,
};

struct blk_mq_ctx;
struct blk_mq_alloc_data;

/*
 * IO Scheduler Operation Function Pointer
 */
struct blk_queue_ops {
	enum iosched_strategy io_scheduler_strategy;
	/* MQ Scheduler Init */
	int (*mq_iosched_init_fn)(struct request_queue *);
	/* MQ Scheduler Deinit */
	void (*mq_iosched_exit_fn)(struct request_queue *);
	/* Prepare for MQ reqeust allocation */
	void (*mq_req_alloc_prep_fn)(struct blk_mq_alloc_data *data,
		unsigned long ioflag, bool fs_submit);
	/* Request init in MQ */
	void (*mq_req_init_fn)(
		struct request_queue *, struct blk_mq_ctx *, struct request *);
	/* Request complete in MQ */
	void (*mq_req_complete_fn)(
		struct request *, struct request_queue *, bool);
	/* Request deinit in MQ */
	void (*mq_req_deinit_fn)(struct request *);
	/* Request insert to MQ */
	void (*mq_req_insert_fn)(struct request *req, struct request_queue *);
	/* Request requeue in MQ */
	void (*mq_req_requeue_fn)(struct request *, struct request_queue *);
	/* Request timeout process in MQ */
	void (*mq_req_timeout_fn)(struct request *);
	/* Get the CTX in MQ */
	struct blk_mq_ctx *(*mq_ctx_get_fn)(struct request_queue *);
	/* Release the CTX in MQ */
	void (*mq_ctx_put_fn)(struct request_queue *);
	/* Get hctx object by request */
	void (*mq_hctx_get_by_req_fn)(
		struct request *, struct blk_mq_hw_ctx **);
	/* Get tag in MQ */
	unsigned int (*mq_tag_get_fn)(struct blk_mq_alloc_data *data);
	/* Release tag in MQ */
	int (*mq_tag_put_fn)(
		struct blk_mq_hw_ctx *, unsigned int, struct request *);
	/* wakeup all threads waiting tag */
	int (*mq_tag_wakeup_all_fn)(struct blk_mq_tags *);
	/* Execute queue function directly in MQ */
	void (*mq_exec_queue_fn)(struct request_queue *);
	/* Run hw queue in MQ */
	void (*mq_run_hw_queue_fn)(struct request_queue *);
	/* Run requeue in MQ */
	void (*mq_run_requeue_fn)(struct request_queue *);
	/* Enable poll */
	void (*blk_poll_enable_fn)(bool *);
	void (*blk_write_throttle_fn)(struct request_queue *, bool);
	/* Dump ioscheduler status */
	void (*blk_status_dump_fn)(struct request_queue *,
		enum blk_dump_scene);
	/* Dump req status in ioscheduler */
	void (*blk_req_dump_fn)(struct request_queue *, enum blk_dump_scene);
	void *scheduler_priv;
	char iosched_name[ELV_NAME_MAX];
};

/*
 * MQ tagset Operation Function Pointer
 */
struct blk_tagset_ops {
	/* MQ tagset init */
	struct blk_mq_tags *(*tagset_init_tags_fn)(unsigned int total_tags,
		unsigned int reserved_tags, unsigned int high_prio_tags,
		int node, int alloc_policy);
	/* MQ tagset free */
	void (*tagset_free_tags_fn)(struct blk_mq_tags *);
	/* iterate all the busy tag in whole tagset */
	void (*tagset_all_tag_busy_iter_fn)(
		struct blk_mq_tags *, busy_tag_iter_fn *, const void *);
	void (*tagset_tag_busy_iter_fn)(
		struct blk_mq_hw_ctx *, busy_iter_fn *, void *);
	void (*tagset_power_off_proc_fn)(struct blk_dev_lld *);
	struct blk_queue_ops *queue_ops;
};

struct bio_delay_stage_config {
	char *stage_name;
	void (*function)(struct bio *bio);
};

struct req_delay_stage_config {
	char *stage_name;
	void (*function)(struct request *req);
};

#ifdef CONFIG_MAS_DEBUG_FS
static inline void mas_blk_rdr_panic(const char *msg)
{
	if (msg)
		pr_err("%s\n", msg);
	rdr_syserr_process_for_ap((u32)MODID_AP_S_PANIC_STORAGE, 0ull, 0ull);
}
#else
static inline void mas_blk_rdr_panic(const char *msg)
{
}
#endif /* CONFIG_MAS_DEBUG_FS */


#endif /* MAS_BLK_INTERNAL_H */
