/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2017-2021. All rights reserved.
 * Description: HKIP protect hooks
 * Date: 2017/08/15
 */

#ifndef __HKIP_H__
#define __HKIP_H__

#include <asm/alternative.h>
#include <asm/cpufeature.h>
#include <linux/sched.h>
#include <linux/threads.h>
#include <linux/types.h>
#include <platform_include/see/hkip_hhee.h>
#include <platform_include/see/hhee_prmem.h>

static inline bool hhee_is_present(void)
{
	bool ret = false;
	return ret;
}

u32 hkip_hvc2(u32, unsigned long);
u32 hkip_hvc3(u32, unsigned long, unsigned long);
u32 hkip_hvc4(u32, unsigned long, unsigned long, unsigned long);

static inline bool hkip_get_bit(const u8 *bits, size_t pos, size_t max)
{
	if (unlikely(pos >= max))
		return false;
	return ((unsigned int)READ_ONCE(bits[pos / 8]) >> (pos % 8)) & 1;
}

static inline void hkip_set_bit(u8 *bits, size_t pos, size_t max, bool value)
{
	if (hkip_get_bit(bits, pos, max) == value)
		return;

	if (unlikely(hkip_hvc4(HKIP_HVC_ROWM_SET_BIT,
			       (unsigned long)(uintptr_t)bits, pos, value)))
		BUG();
}

static inline bool __hkip_get_task_bit(const u8 *bits, struct task_struct *task)
{
	return hkip_get_bit(bits, task_pid_nr(task), PID_MAX_DEFAULT);
}

static inline bool hkip_get_task_bit(const u8 *bits, struct task_struct *task,
				     bool def_value)
{
	pid_t pid = task_pid_nr(task);
	if (pid != 0)
		return hkip_get_bit(bits, pid, PID_MAX_DEFAULT);
	return def_value;
}

static inline void hkip_set_task_bit(u8 *bits, struct task_struct *task,
				     bool value)
{
	pid_t pid = task_pid_nr(task);
	if (pid != 0)
		hkip_set_bit(bits, pid, PID_MAX_DEFAULT, value);
}

static inline bool __hkip_get_current_bit(const u8 *bits)
{
	return __hkip_get_task_bit(bits, current);
}

static inline bool hkip_get_current_bit(const u8 *bits, bool def_value)
{
	return hkip_get_task_bit(bits, current, def_value);
}

static inline void hkip_set_current_bit(u8 *bits, bool value)
{
	hkip_set_task_bit(bits, current, value);
}

struct cred;


#define hkip_set_fs(fs) ((void)(fs))

#ifndef CONFIG_HKIP_PROTECT_CRED
static inline int hkip_check_uid_root(void)
{
	return 0;
}
static inline int hkip_check_gid_root(void)
{
	return 0;
}
static inline int hkip_check_xid_root(void)
{
	return 0;
}
static inline void hkip_update_xid_root(const struct cred *creds)
{
}
#endif /* CONFIG_HKIP_PROTECT_CRED */

static inline void hkip_init_task(struct task_struct *task)
{
}

static inline int hkip_register_xo(const uintptr_t base, size_t size)
{
	return 0;
}

#ifdef CONFIG_HKIP_MAP_KERNEL_PROTECT
void hkip_set_pgtable_protect_addr(void);
int hkip_get_pgtable_protect_addr(phys_addr_t *start, phys_addr_t *end);
unsigned long hkip_pgtable_phys_alloc(unsigned long size, unsigned long align);
#else
static inline void hkip_set_pgtable_protect_addr(void) {}

static inline int hkip_get_pgtable_protect_addr(phys_addr_t *start, phys_addr_t *end)
{
	(void)start;
	(void)end;
	return 0;
}

static inline unsigned long hkip_pgtable_phys_alloc(unsigned long size, unsigned long align)
{
	return 0;
}
#endif /* CONFIG_HKIP_MAP_KERNEL_PROTECT */

#ifdef CONFIG_HHEE_DEBUG
void hkip_get_stage2_pgtab(unsigned long addr);
#else
static inline void hkip_get_stage2_pgtab(unsigned long addr) {}
#endif /* CONFIG_HHEE_DEBUG */

#endif /* __HKIP_H__ */
